@@ -1,26 +1,32 @@
-from copy import copy
-from inspect import isclass
-from typing import TypeVar, Union, Tuple, Callable  # do not import Type for compatibility with earlier python 3.5
 from warnings import warn
 import sys
 
+try:  # python 3.5+
+    from typing import TypeVar, Union, Tuple, Callable, Any
+    try: # old python 3.5
+        from typing import Type
+    except ImportError:
+        pass
+    T = TypeVar('T')
+except ImportError:
+    pass
+
 from mini_lambda.base import get_repr, _PRECEDENCE_BITWISE_AND, _PRECEDENCE_BITWISE_OR, _PRECEDENCE_BITWISE_XOR, \
     _PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF, evaluate, _PRECEDENCE_EXPONENTIATION, \
-    _PRECEDENCE_POS_NEG_BITWISE_NOT, _get_root_var
-from mini_lambda.generated import _LambdaExpressionGenerated, FunctionDefinitionError
+    _PRECEDENCE_POS_NEG_BITWISE_NOT, _get_root_var, FunctionDefinitionError
+from mini_lambda.generated_magic import _LambdaExpressionGenerated
 
-T = TypeVar('T')
 
 this_module = sys.modules[__name__]
 
 
-class _LambdaExpression(_LambdaExpressionGenerated):
+class LambdaExpression(_LambdaExpressionGenerated):
     """
     Represents a lambda function.
     * It can be evaluated by calling the 'evaluate' method. In such case it will apply its inner function (self._fun)
     on the inputs provided.
     * It can be transformed into a normal function (a callable object) with `self.as_function()`.
-    * Any other method called on this object will create a new _LambdaExpression instance with that same method
+    * Any other method called on this object will create a new LambdaExpression instance with that same method
     stacked on top of the current inner function. For example x.foo() will return a new expression, that, when
     executed, will call x._fun first and then perform .foo() on its results.
 
@@ -41,18 +47,18 @@ class _LambdaExpression(_LambdaExpressionGenerated):
 
     class LambdaFunction:
         """ A view on a lambda expression, that is only capable of evaluating but is able to do it in a more friendly
-        way: simply calling it with arguments is ok, instead of calling .evaluate() like for the _LambdaExpression.
+        way: simply calling it with arguments is ok, instead of calling .evaluate() like for the LambdaExpression.
         Another side effect is that this object is representable: you can call str() on it. You may return to the
         associated expression """
 
         def __init__(self, expression):
             """
-            Constructor from a mandatory existing _LambdaExpression.
+            Constructor from a mandatory existing LambdaExpression.
             :param expression:
             """
             self.expression = expression
 
-        def __call__(self, *args, **kwargs):
+        def __call__(self, arg):
             """
             Calling this object is actually evaluating the inner expression with the given arguments.
             So it behaves like a normal function.
@@ -61,7 +67,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
             :param kwargs:
             :return:
             """
-            return self.expression.evaluate(*args, **kwargs)
+            return self.expression.evaluate(arg)
 
         def as_expression(self):
             """
@@ -73,6 +79,9 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         def __str__(self):
             return self.expression._str_expr
 
+        def __repr__(self):
+            return "<LambdaFunction: %s>" % str(self)
+
     def as_function(self):
         """
         freezes this expression so that it can be called directly, in other words that calling it actually calls
@@ -80,12 +89,12 @@ class _LambdaExpression(_LambdaExpressionGenerated):
 
         :return: a callable object created by freezing this input expression
         """
-        return _LambdaExpression.LambdaFunction(self)
+        return LambdaExpression.LambdaFunction(self)
 
     # Special case: List comprehensions
     def __iter__(self):
         """ This method is forbidden so that we can detect and prevent usages in list/set/tuple/dict comprehensions """
-        raise FunctionDefinitionError('__iter__ cannot be used with a _LambdaExpression. If you meant to use iter() '
+        raise FunctionDefinitionError('__iter__ cannot be used with a LambdaExpression. If you meant to use iter() '
                                       'explicitly, please use the replacement method Iter() provided in mini_lambda. '
                                       'Otherwise you probably ended in this exception because there is a list/set/'
                                       'tuple/dict comprehension in your expression, such as [i for i in x]. These are '
@@ -96,9 +105,9 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         """ This magic method is forbidden because python casts the result before returning """
 
         # see https://stackoverflow.com/questions/37140933/custom-chained-comparisons
-        raise FunctionDefinitionError('__bool__ cannot be used with a _LambdaExpression. Please use '
+        raise FunctionDefinitionError('__bool__ cannot be used with a LambdaExpression. Please use '
                                       'the Bool() method provided in mini_lambda instead. If you got this error message'
-                                      ' but you do not call bool(x) or x.__bool__() in your _LambdaExpression then you '
+                                      ' but you do not call bool(x) or x.__bool__() in your LambdaExpression then you '
                                       'probably'
                                       '\n * use a chained comparison such as 0 < x < 1, in such case please consider '
                                       'rewriting it without chained comparisons, such as in (0 < x) & (x < 1). '
@@ -111,10 +120,10 @@ class _LambdaExpression(_LambdaExpressionGenerated):
 
     def __and__(self, other):
         """
-        A logical AND between this _LambdaExpression and something else. The other part can either be
+        A logical AND between this LambdaExpression and something else. The other part can either be
         * a scalar
         * a callable
-        * a _LambdaExpression
+        * a LambdaExpression
 
         :param other:
         :return:
@@ -122,7 +131,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         if not callable(other):
             # then the other part has already been evaluated, since this operator is not a short-circuit like the 'and'
             # keyword. This is probably an error from the developer ? Warn him/her
-            warn("One of the sides of an '&' operator is not a _LambdaExpression and therefore will always have the "
+            warn("One of the sides of an '&' operator is not a LambdaExpression and therefore will always have the "
                  "same value, whatever the input. This is most probably a mistake in the expression.")
             if not other:
                 # the other part is False, there will never be a need to evaluate.
@@ -134,7 +143,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
             # check that both work on the same variable
             root_var, _ = _get_root_var(self, other)
 
-            # create a new _LambdaExpression able to evaluate both sides with short-circuit capability
+            # create a new LambdaExpression able to evaluate both sides with short-circuit capability
             def evaluate_both_inner_functions_and_combine(input):
                 # first evaluate self
                 left = self.evaluate(input)
@@ -146,17 +155,17 @@ class _LambdaExpression(_LambdaExpressionGenerated):
                     return bool(evaluate(other, input))
 
             string_expr = get_repr(self, _PRECEDENCE_BITWISE_AND) + ' & ' + get_repr(other, _PRECEDENCE_BITWISE_AND)
-            return _LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
-                                     precedence_level=_PRECEDENCE_BITWISE_AND,
-                                     str_expr=string_expr,
-                                     root_var=root_var)
+            return LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
+                                    precedence_level=_PRECEDENCE_BITWISE_AND,
+                                    str_expr=string_expr,
+                                    root_var=root_var, repr_on=self.repr_on)
 
     def __or__(self, other):
         """
-        A logical OR between this _LambdaExpression and something else. The other part can either be
+        A logical OR between this LambdaExpression and something else. The other part can either be
         * a scalar
         * a callable
-        * a _LambdaExpression
+        * a LambdaExpression
 
         A special operation can be performed by doing '| _'. This will 'close' the expression and return a callable
         view of it
@@ -172,7 +181,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         if not callable(other):
             # then the other part has already been evaluated, since this operator is not a short-circuit like the 'or'
             # keyword. This is probably an error from the developer ? Warn him/her
-            warn("One of the sides of an '|' operator is not a _LambdaExpression and therefore will always have the same"
+            warn("One of the sides of an '|' operator is not a LambdaExpression and therefore will always have the same"
                  " value, whatever the input. This is most probably a mistake in the expression.")
             if other:
                 # the other part is True, there will never be a need to evaluate.
@@ -184,7 +193,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
             # check that both work on the same variable
             root_var, _ = _get_root_var(self, other)
 
-            # create a new _LambdaExpression able to evaluate both sides with short-circuit capability
+            # create a new LambdaExpression able to evaluate both sides with short-circuit capability
             def evaluate_both_inner_functions_and_combine(input):
                 # first evaluate self
                 left = self.evaluate(input)
@@ -196,17 +205,17 @@ class _LambdaExpression(_LambdaExpressionGenerated):
                     return bool(evaluate(other, input))
 
             string_expr = get_repr(self, _PRECEDENCE_BITWISE_OR) + ' | ' + get_repr(other, _PRECEDENCE_BITWISE_OR)
-            return _LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
-                                     precedence_level=_PRECEDENCE_BITWISE_OR,
-                                     str_expr=string_expr,
-                                     root_var=root_var)
+            return LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
+                                    precedence_level=_PRECEDENCE_BITWISE_OR,
+                                    str_expr=string_expr,
+                                    root_var=root_var, repr_on=self.repr_on)
 
     def __xor__(self, other):
         """
-        A logical XOR between this _LambdaExpression and something else. The other part can either be
+        A logical XOR between this LambdaExpression and something else. The other part can either be
         * a scalar
         * a callable
-        * a _LambdaExpression
+        * a LambdaExpression
 
         :param other:
         :return:
@@ -214,7 +223,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         if not callable(other):
             # then the other part has already been evaluated, since this operator is not a short-circuit like the 'or'
             # keyword. This is probably an error from the developer ? Warn him/her
-            warn("One of the sides of an '^' operator is not a _LambdaExpression and therefore will always have the "
+            warn("One of the sides of an '^' operator is not a LambdaExpression and therefore will always have the "
                  "same value, whatever the input. This is most probably a mistake in the expression.")
             if other:
                 # the other part is True, so this becomes a Not expression of self (The Not function is created later)
@@ -226,7 +235,7 @@ class _LambdaExpression(_LambdaExpressionGenerated):
             # check that both work on the same variable
             root_var, _ = _get_root_var(self, other)
 
-            # create a new _LambdaExpression able to evaluate both sides
+            # create a new LambdaExpression able to evaluate both sides
             def evaluate_both_inner_functions_and_combine(input):
                 # first evaluate self
                 left = self.evaluate(input)
@@ -237,56 +246,56 @@ class _LambdaExpression(_LambdaExpressionGenerated):
                 return (left and not right) or (not left and right)
 
             string_expr = get_repr(self, _PRECEDENCE_BITWISE_XOR) + ' ^ ' + get_repr(other, _PRECEDENCE_BITWISE_XOR)
-            return _LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
-                                     precedence_level=_PRECEDENCE_BITWISE_XOR,
-                                     str_expr=string_expr,
-                                     root_var=root_var)
+            return LambdaExpression(fun=evaluate_both_inner_functions_and_combine,
+                                    precedence_level=_PRECEDENCE_BITWISE_XOR,
+                                    str_expr=string_expr,
+                                    root_var=root_var, repr_on=self.repr_on)
 
     def not_(self):
-        """ Returns a new _LambdaExpression performing 'not x' on the result of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'not x' on the result of this expression's evaluation """
         def __not(x):
             return not x
 
         return self.add_unbound_method_to_stack(__not)
 
     def any_(self):
-        """ Returns a new _LambdaExpression performing 'any(x)' on the result of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'any(x)' on the result of this expression's evaluation """
         return self.add_unbound_method_to_stack(any)
 
     def all_(self):
-        """ Returns a new _LambdaExpression performing 'all(x)' on the result of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'all(x)' on the result of this expression's evaluation """
         return self.add_unbound_method_to_stack(all)
 
     # Special case: indexing
     def __index__(self):
         """ This magic method is forbidden because python casts the result before returning """
-        raise FunctionDefinitionError('It is not currently possible to use a _LambdaExpression as an index, e.g.'
+        raise FunctionDefinitionError('It is not currently possible to use a LambdaExpression as an index, e.g.'
                                       'o[x], where x is the input variable. Instead, please use the equivalent operator'
                                       ' provided in mini_lambda : Get(o, x)')
 
     # Special case: membership testing
     def __contains__(self, item):
         """ This magic method is forbidden because python casts the result before returning """
-        raise FunctionDefinitionError('membership operators in/ not in cannot be used with a _LambdaExpression because '
+        raise FunctionDefinitionError('membership operators in/ not in cannot be used with a LambdaExpression because '
                                       'python casts the result as a bool. Therefore this __contains__ method is '
                                       'forbidden. Alternate x.contains() and x.is_in() methods are provided to replace it, '
                                       'as well as an In() method')
 
     def is_in(self, container):
-        """ Returns a new _LambdaExpression performing 'res in container' where res is the result of evaluating self"""
+        """ Returns a new LambdaExpression performing 'res in container' where res is the result of evaluating self"""
         def _item_in(x):
             return x in container
         return self.add_unbound_method_to_stack(_item_in)
 
     def contains(self, item):
-        """ Returns a new _LambdaExpression performing 'item in res' on the result of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'item in res' on the result of this expression's evaluation """
         def _item_in(x):
             return item in x
         return self.add_unbound_method_to_stack(_item_in)
 
     # Special case for the string representation
     def __getattr__(self, name):
-        """ Returns a new _LambdaExpression performing 'getattr(<r>, *args)' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'getattr(<r>, *args)' on the result <r> of this expression's evaluation """
         root_var, _ = _get_root_var(self, name)
         def ___getattr__(input):
             # first evaluate the inner function
@@ -297,11 +306,11 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         # return a new LambdaExpression of the same type than self, with the new function as inner function
         string_expr = get_repr(self, _PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF) + '.' + name
         return type(self)(fun=___getattr__, precedence_level=_PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF,
-                          str_expr=string_expr, root_var=root_var)
+                          str_expr=string_expr, root_var=root_var, repr_on=self.repr_on)
 
     # Special case for the string representation
     def __call__(self, *args, **kwargs):
-        """ Returns a new _LambdaExpression performing '<r>.__call__(*args, **kwargs)' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing '<r>.__call__(*args, **kwargs)' on the result <r> of this expression's evaluation """
         # return self.add_bound_method_to_stack('__call__', *args, **kwargs)
         root_var, _ = _get_root_var(self, *args, **kwargs)
         def ___call__(input):
@@ -318,11 +327,11 @@ class _LambdaExpression(_LambdaExpressionGenerated):
                       + (', ' if (len(args) > 0 and len(kwargs) > 0) else '')\
                       + ', '.join([arg_name + '=' + get_repr(arg, None) for arg_name, arg in kwargs.items()]) + ')'
         return type(self)(fun=___call__, precedence_level=_PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF,
-                          str_expr=string_expr, root_var=root_var)
+                          str_expr=string_expr, root_var=root_var, repr_on=self.repr_on)
 
     # Special case for the string representation
     def __getitem__(self, key):
-        """ Returns a new _LambdaExpression performing '<r>.__getitem__(*args)' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing '<r>.__getitem__(*args)' on the result <r> of this expression's evaluation """
         # return self.add_bound_method_to_stack('__getitem__', *args)
         root_var, _ = _get_root_var(self, key)
         def ___getitem__(input):
@@ -336,11 +345,11 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         string_expr = get_repr(self, _PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF) \
                       + '[' + get_repr(key, None) + ']'
         return type(self)(fun=___getitem__, precedence_level=_PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF,
-                          str_expr=string_expr, root_var=root_var)
+                          str_expr=string_expr, root_var=root_var, repr_on=self.repr_on)
 
     # Special case for string representation because pow is asymetric in precedence
     def __pow__(self, other):
-        """ Returns a new _LambdaExpression performing '<r> ** other' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing '<r> ** other' on the result <r> of this expression's evaluation """
         root_var, _ = _get_root_var(self, other)
         def ___pow__(input):
             # first evaluate the inner function
@@ -351,11 +360,12 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         # return a new LambdaExpression of the same type than self, with the new function as inner function
         string_expr = get_repr(self, _PRECEDENCE_EXPONENTIATION) + ' ** ' \
                       + get_repr(other, _PRECEDENCE_POS_NEG_BITWISE_NOT)
-        return type(self)(fun=___pow__, precedence_level=13, str_expr=string_expr, root_var=root_var)
+        return type(self)(fun=___pow__, precedence_level=13, str_expr=string_expr, root_var=root_var,
+                          repr_on=self.repr_on)
 
     # Special case for string representation because pow is asymetric in precedence
     def __rpow__(self, other):
-        """ Returns a new _LambdaExpression performing 'other ** <r>' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing 'other ** <r>' on the result <r> of this expression's evaluation """
         root_var, _ = _get_root_var(self, other)
         def ___rpow__(input):
             # first evaluate the inner function
@@ -366,11 +376,12 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         # return a new LambdaExpression of the same type than self, with the new function as inner function
         string_expr = get_repr(other, _PRECEDENCE_EXPONENTIATION) + ' ** ' \
                       + get_repr(self, _PRECEDENCE_POS_NEG_BITWISE_NOT)
-        return type(self)(fun=___rpow__, precedence_level=13, str_expr=string_expr, root_var=root_var)
+        return type(self)(fun=___rpow__, precedence_level=13, str_expr=string_expr, root_var=root_var,
+                          repr_on=self.repr_on)
 
     # Special case : unbound function call but with left/right
     def __divmod__(self, other):
-        """ Returns a new _LambdaExpression performing '<r>.__divmod__(*args)' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing '<r>.__divmod__(*args)' on the result <r> of this expression's evaluation """
         # return self.add_bound_method_to_stack('__divmod__', *args)
         root_var, _ = _get_root_var(self, other)
         def ___divmod__(input):
@@ -383,11 +394,11 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         # Note: we use precedence=None for coma-separated items inside the parenthesis
         string_expr = 'divmod(' + get_repr(self, None) + ', ' + get_repr(other, None) + ')'
         return type(self)(fun=___divmod__, precedence_level=_PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF,
-                          str_expr=string_expr, root_var=root_var)
+                          str_expr=string_expr, root_var=root_var, repr_on=self.repr_on)
 
     # Special case : unbound function call but with left/right
     def __rdivmod__(self, other):
-        """ Returns a new _LambdaExpression performing '<r>.__rdivmod__(*args)' on the result <r> of this expression's evaluation """
+        """ Returns a new LambdaExpression performing '<r>.__rdivmod__(*args)' on the result <r> of this expression's evaluation """
         # return self.add_bound_method_to_stack('__rdivmod__', *args)
         root_var, _ = _get_root_var(self, other)
         def ___rdivmod__(input):
@@ -400,17 +411,19 @@ class _LambdaExpression(_LambdaExpressionGenerated):
         # Note: we use precedence=None for coma-separated items inside the parenthesis
         string_expr = 'divmod(' + get_repr(other, None) + ', ' + get_repr(self, None) + ')'
         return type(self)(fun=___rdivmod__, precedence_level=_PRECEDENCE_SUBSCRIPTION_SLICING_CALL_ATTRREF,
-                          str_expr=string_expr, root_var=root_var)
+                          str_expr=string_expr, root_var=root_var, repr_on=self.repr_on)
 
     # special case: format(x, args) does not work but x.format() works
     def __format__(self, *args):
         """
-        This magic method can not be used on an _LambdaExpression, because unfortunately python checks the
+        This magic method can not be used on an LambdaExpression, because unfortunately python checks the
         result type and does not allow it to be a custom type.
         """
-        raise FunctionDefinitionError('__format__ is not supported by _LambdaExpression, since python raises an'
+        raise FunctionDefinitionError('__format__ is not supported by LambdaExpression, since python raises an'
                                       ' error when its output is not directly an object of the type it expects.'
-                                      'Please either use the equivalent x.format() method, or the Format method provided'
+                                      'Please either use the equivalent x.format() method if your variable is the '
+                                      'string template, or the `Format` (for `format`) or `Str.format` '
+                                      '(for `str.format`) methods provided'
                                       ' at mini_lambda package level.If you did not use __format__ in your expression, '
                                       'you probably used a standard method such as math.log(x) instead of a method '
                                       ' converted to mini_lambda such as Log(x). Please check the documentation for '
@@ -418,9 +431,10 @@ class _LambdaExpression(_LambdaExpressionGenerated):
 
 
 # Special case: 'not' is not a function
-def Not(expression: _LambdaExpression):
+def Not(expression  # type: LambdaExpression
+        ):
     """
-    Equivalent of 'not x' for a _LambdaExpression.
+    Equivalent of 'not x' for a LambdaExpression.
 
     :param expression:
     :return:
@@ -441,7 +455,7 @@ def And(a, b):
     :param b: right operand
     :return: expression evaluating the and combination
     """
-    return _LambdaExpression._get_expression_for_method_with_args(_and, a, b)
+    return LambdaExpression._get_expression_for_method_with_args(_and, a, b)
 
 
 def _or(a, b):
@@ -456,7 +470,7 @@ def Or(a, b):
     :param b: right operand
     :return: expression evaluating the or combination
     """
-    return _LambdaExpression._get_expression_for_method_with_args(_or, a, b)
+    return LambdaExpression._get_expression_for_method_with_args(_or, a, b)
 
 
 # Special case: we do not want to use format() but type(value).format. So we override the generated method
@@ -469,7 +483,7 @@ def Format(value, *args, **kwargs):
     :param kwargs:
     :return:
     """
-    return _LambdaExpression._get_expression_for_method_with_args(type(value).format, value, *args, **kwargs)
+    return LambdaExpression._get_expression_for_method_with_args(type(value).format, value, *args, **kwargs)
 
 
 # Special case: the unbound method 'getitem' does not exist, and we want type(value).__getitem__
@@ -486,7 +500,7 @@ def Get(container, key):
     :param key:
     :return:
     """
-    return _LambdaExpression._get_expression_for_method_with_args(type(container).__getitem__, container, key)
+    return LambdaExpression._get_expression_for_method_with_args(type(container).__getitem__, container, key)
 
 
 # ************** All of these could be generated
@@ -504,7 +518,7 @@ def In(item, container):
     :param container:
     :return:
     """
-    return _LambdaExpression._get_expression_for_method_with_args(_is_in, item, container)
+    return LambdaExpression._get_expression_for_method_with_args(_is_in, item, container)
 
 
 def Slice(*args, **kwargs):
@@ -515,13 +529,14 @@ def Slice(*args, **kwargs):
     :param kwargs:
     :return:
     """
-    return _LambdaExpression._get_expression_for_method_with_args(slice, *args, **kwargs)
+    return LambdaExpression._get_expression_for_method_with_args(slice, *args, **kwargs)
 
 # ************************
 
 
-def _(*expressions: _LambdaExpression) -> Union[_LambdaExpression.LambdaFunction,
-                                               Tuple[_LambdaExpression.LambdaFunction, ...]]:
+def _(*expressions  # type: LambdaExpression
+      ):
+    # type: (...) -> Union[LambdaExpression.LambdaFunction, Tuple[LambdaExpression.LambdaFunction, ...]]
     """
     'Closes' one or several lambda expressions, in other words, transforms them into callable functions.
 
@@ -540,7 +555,10 @@ F = _
 """ Alias for '_' """
 
 
-def InputVar(symbol: str = None, typ: 'Type[T]' = None) -> Union[T, _LambdaExpression]:
+def InputVar(symbol=None,  # type: str
+             typ=None      # type: Type[T]
+             ):
+    # type: (...) -> Union[T, LambdaExpression]
     """
     Creates a variable to use in validator expression. The optional `typ` argument may be used to get a variable with
     appropriate syntactic completion from your IDE, but is not used for anything else.
@@ -553,10 +571,13 @@ def InputVar(symbol: str = None, typ: 'Type[T]' = None) -> Union[T, _LambdaExpre
     if not isinstance(symbol, str):
         raise TypeError("symbol should be a string. It is recommended to use a very small string that is identical "
                         "to the python variable name, for example  s = InputVar('s')")
-    return _LambdaExpression(symbol)
+    return LambdaExpression(symbol)
 
 
-def Constant(value: T, name: str = None) -> Union[T, _LambdaExpression]:
+def Constant(value,     # type: T
+             name=None  # type: str
+             ):
+    # type: (...) -> Union[T, LambdaExpression]
     """
     Creates a constant expression. This is useful when
     * you want to use a method on an object that is not an expression, as in 'toto'.prefix(x) where x is an expression.
@@ -571,7 +592,7 @@ def Constant(value: T, name: str = None) -> Union[T, _LambdaExpression]:
     :param name:
     :return:
     """
-    return _LambdaExpression.constant(value, name)
+    return LambdaExpression.constant(value, name)
 
 
 C = Constant
@@ -582,7 +603,10 @@ make_lambda_friendly = Constant
 """ Alias for 'Constant' """
 
 
-def make_lambda_friendly_class(typ: 'Type[T]', name: str = None) -> 'Union[Type[T], _LambdaExpression]':
+def make_lambda_friendly_class(typ,       # type: Type[T]
+                               name=None  # type: str
+                               ):
+    # type: (...) -> Union[Type[T], LambdaExpression]
     """
     Utility method to transform a standard class into a class usable inside lambda expressions, as in
     DDataFrame = C(DataFrame), so as to be able to use it in expressions
@@ -594,7 +618,10 @@ def make_lambda_friendly_class(typ: 'Type[T]', name: str = None) -> 'Union[Type[
     return Constant(typ, name=name)
 
 
-def make_lambda_friendly_method(method: Callable, name: str = None) -> _LambdaExpression:
+def make_lambda_friendly_method(method,    # type: Callable
+                                name=None  # type: str
+                                ):
+    # type: (...) -> LambdaExpression
     """
     Utility method to transform any method whatever their signature (positional and/or keyword arguments,
     variable-length included) into a method usable inside lambda expressions, even if some of the arguments are
@@ -685,3 +712,31 @@ def make_lambda_friendly_method(method: Callable, name: str = None) -> _LambdaEx
     :return:
     """
     return Constant(method, name)
+
+
+def is_mini_lambda_expr(f  # type: Union[LambdaExpression, Any]
+                        ):
+    """
+    Returns `True` if f is a mini-lambda expression, False otherwise
+
+    :param f:
+    :return:
+    """
+    try:
+        return issubclass(f.__class__, LambdaExpression)
+    except (AttributeError, TypeError):
+        return False
+
+
+def as_function(f  # type: Union[LambdaExpression, Any]
+                ):
+    """
+    Returns `f.as_function()` if f is a mini-lambda expression, returns f otherwise.
+
+    :param f:
+    :return:
+    """
+    if is_mini_lambda_expr(f):
+        return f.as_function()
+    else:
+        return f
